@@ -1,4 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,6 +16,7 @@
 import datetime
 from heat.common import exception
 from heat.openstack.common import log as logging
+from heat.openstack.common.gettextutils import _
 from heat.openstack.common import timeutils
 from heat.engine import timestamp
 from heat.db import api as db_api
@@ -30,11 +30,15 @@ class WatchRule(object):
     WATCH_STATES = (
         ALARM,
         NORMAL,
-        NODATA
+        NODATA,
+        SUSPENDED,
+        CEILOMETER_CONTROLLED,
     ) = (
         rpc_api.WATCH_STATE_ALARM,
         rpc_api.WATCH_STATE_OK,
-        rpc_api.WATCH_STATE_NODATA
+        rpc_api.WATCH_STATE_NODATA,
+        rpc_api.WATCH_STATE_SUSPENDED,
+        rpc_api.WATCH_STATE_CEILOMETER_CONTROLLED,
     )
     ACTION_MAP = {ALARM: 'AlarmActions',
                   NORMAL: 'OKActions',
@@ -52,7 +56,12 @@ class WatchRule(object):
         self.state = state
         self.rule = rule
         self.stack_id = stack_id
-        self.timeperiod = datetime.timedelta(seconds=int(rule['Period']))
+        period = 0
+        if 'Period' in rule:
+            period = int(rule['Period'])
+        elif 'period' in rule:
+            period = int(rule['period'])
+        self.timeperiod = datetime.timedelta(seconds=period)
         self.id = wid
         self.watch_data = watch_data
         self.last_evaluated = last_evaluated
@@ -66,8 +75,9 @@ class WatchRule(object):
             try:
                 watch = db_api.watch_rule_get_by_name(context, watch_name)
             except Exception as ex:
-                logger.warn('WatchRule.load (%s) db error %s' %
-                            (watch_name, str(ex)))
+                logger.warn(_('WatchRule.load (%(watch_name)s) db error '
+                            '%(ex)s') % {
+                                'watch_name': watch_name, 'ex': str(ex)})
         if watch is None:
             raise exception.WatchRuleNotFound(watch_name=watch_name)
         else:
@@ -99,6 +109,13 @@ class WatchRule(object):
         else:
             db_api.watch_rule_update(self.context, self.id, wr_values)
 
+    def destroy(self):
+        '''
+        Delete the watchrule from the database.
+        '''
+        if self.id:
+            db_api.watch_rule_delete(self.context, self.id)
+
     def do_data_cmp(self, data, threshold):
         op = self.rule['ComparisonOperator']
         if op == 'GreaterThanThreshold':
@@ -193,7 +210,7 @@ class WatchRule(object):
         data = 0
         for d in self.watch_data:
             if d.created_at < self.now - self.timeperiod:
-                logger.debug('ignoring %s' % str(d.data))
+                logger.debug(_('ignoring %s') % str(d.data))
                 continue
             data = data + float(d.data[self.rule['MetricName']]['Value'])
 
@@ -208,52 +225,91 @@ class WatchRule(object):
         return fn()
 
     def evaluate(self):
+        if self.state == self.SUSPENDED:
+            return []
         # has enough time progressed to run the rule
         self.now = timeutils.utcnow()
         if self.now < (self.last_evaluated + self.timeperiod):
             return []
         return self.run_rule()
 
+    def get_details(self):
+        return {'alarm': self.name,
+                'state': self.state}
+
     def run_rule(self):
         new_state = self.get_alarm_state()
-
-        actions = []
-        if new_state != self.state:
-            actions = self.rule_actions(new_state)
-            self.state = new_state
+        actions = self.rule_actions(new_state)
+        self.state = new_state
 
         self.last_evaluated = self.now
         self.store()
         return actions
 
     def rule_actions(self, new_state):
-        logger.info('WATCH: stack:%s, watch_name:%s %s',
-                    self.stack_id, self.name, new_state)
+        logger.info(_('WATCH: stack:%(stack)s, watch_name:%(watch_name)s, '
+                      'new_state:%(new_state)s'), {'stack': self.stack_id,
+                                                   'watch_name': self.name,
+                                                   'new_state': new_state})
         actions = []
-        if not self.ACTION_MAP[new_state] in self.rule:
-            logger.info('no action for new state %s',
+        if self.ACTION_MAP[new_state] not in self.rule:
+            logger.info(_('no action for new state %s'),
                         new_state)
         else:
             s = db_api.stack_get(self.context, self.stack_id)
-            if s and s.status in (parser.Stack.CREATE_COMPLETE,
-                                  parser.Stack.UPDATE_COMPLETE):
-                stack = parser.Stack.load(self.context, stack=s)
-                for a in self.rule[self.ACTION_MAP[new_state]]:
-                    actions.append(stack[a].alarm)
+            stack = parser.Stack.load(self.context, stack=s)
+            if (stack.action != stack.DELETE
+                    and stack.status == stack.COMPLETE):
+                for refid in self.rule[self.ACTION_MAP[new_state]]:
+                    actions.append(stack.resource_by_refid(refid).signal)
             else:
-                logger.warning("Could not process watch state %s for stack" %
-                               new_state)
+                logger.warning(_("Could not process watch state %s for stack")
+                               % new_state)
         return actions
 
+    def _to_ceilometer(self, data):
+        from heat.engine import clients
+        clients = clients.Clients(self.context)
+        sample = {}
+        sample['meter_type'] = 'gauge'
+
+        for k, d in iter(data.items()):
+            if k == 'Namespace':
+                continue
+            sample['meter_name'] = k
+            sample['sample_volume'] = d['Value']
+            sample['meter_unit'] = d['Unit']
+            dims = d.get('Dimensions', {})
+            if isinstance(dims, list):
+                dims = dims[0]
+            sample['resource_metadata'] = dims
+            sample['resource_id'] = dims.get('InstanceId')
+            logger.debug(_('new sample:%(k)s data:%(sample)s') % {
+                         'k': k, 'sample': sample})
+            clients.ceilometer().samples.create(**sample)
+
     def create_watch_data(self, data):
-        if not self.rule['MetricName'] in data:
+        if self.state == self.CEILOMETER_CONTROLLED:
+            # this is a short term measure for those that have cfn-push-stats
+            # within their templates, but want to use Ceilometer alarms.
+
+            self._to_ceilometer(data)
+            return
+
+        if self.state == self.SUSPENDED:
+            logger.debug(_('Ignoring metric data for %s, SUSPENDED state')
+                         % self.name)
+            return []
+
+        if self.rule['MetricName'] not in data:
             # Our simplified cloudwatch implementation only expects a single
             # Metric associated with each alarm, but some cfn-push-stats
             # options, e.g --haproxy try to push multiple metrics when we
             # actually only care about one (the one we're alarming on)
             # so just ignore any data which doesn't contain MetricName
-            logger.debug('Ignoring metric data (only accept %s) : %s' %
-                        (self.rule['MetricName'], data))
+            logger.debug(_('Ignoring metric data (only accept %(metric)s) '
+                         ': %(data)s') % {
+                             'metric': self.rule['MetricName'], 'data': data})
             return
 
         watch_data = {
@@ -261,7 +317,18 @@ class WatchRule(object):
             'watch_rule_id': self.id
         }
         wd = db_api.watch_data_create(None, watch_data)
-        logger.debug('new watch:%s data:%s' % (self.name, str(wd.data)))
+        logger.debug(_('new watch:%(name)s data:%(data)s')
+                     % {'name': self.name, 'data': str(wd.data)})
+
+    def state_set(self, state):
+        '''
+        Persistently store the watch state
+        '''
+        if state not in self.WATCH_STATES:
+            raise ValueError(_("Invalid watch state %s") % state)
+
+        self.state = state
+        self.store()
 
     def set_watch_state(self, state):
         '''
@@ -270,15 +337,55 @@ class WatchRule(object):
         '''
 
         if state not in self.WATCH_STATES:
-            raise ValueError('Unknown watch state %s' % state)
+            raise ValueError(_('Unknown watch state %s') % state)
 
         actions = []
         if state != self.state:
             actions = self.rule_actions(state)
             if actions:
-                logger.debug("Overriding state %s for watch %s with %s" %
-                            (self.state, self.name, state))
+                logger.debug(_("Overriding state %(self_state)s for watch "
+                             "%(name)s with %(state)s") % {
+                                 'self_state': self.state, 'name': self.name,
+                                 'state': state})
             else:
-                logger.warning("Unable to override state %s for watch %s" %
-                              (self.state, self.name))
+                logger.warning(_("Unable to override state %(state)s for "
+                               "watch %(name)s") % {
+                                   'state': self.state, 'name': self.name})
         return actions
+
+
+def rule_can_use_sample(wr, stats_data):
+    def match_dimesions(rule, data):
+        for k, v in iter(rule.items()):
+            if k not in data:
+                return False
+            elif v != data[k]:
+                return False
+        return True
+
+    if wr.state == WatchRule.SUSPENDED:
+        return False
+    if wr.state == WatchRule.CEILOMETER_CONTROLLED:
+        metric = wr.rule['meter_name']
+        rule_dims = {}
+        for k, v in iter(wr.rule.get('matching_metadata', {}).items()):
+            name = k.split('.')[-1]
+            rule_dims[name] = v
+    else:
+        metric = wr.rule['MetricName']
+        rule_dims = dict((d['Name'], d['Value'])
+                         for d in wr.rule.get('Dimensions', []))
+
+    if metric not in stats_data:
+        return False
+
+    for k, v in iter(stats_data.items()):
+        if k == 'Namespace':
+            continue
+        if k == metric:
+            data_dims = v.get('Dimensions', {})
+            if isinstance(data_dims, list):
+                data_dims = data_dims[0]
+            if match_dimesions(rule_dims, data_dims):
+                return True
+    return False
